#! --PERL--
##
##
## This module is part of ML and provides some tools

package mail;

use Conf;
use Log;
use Language;


## RCS identification.
#my $id = '@(#)$Id: mail.pl,v 1.1.1.1 1998/09/18 09:16:06 sympa Exp $';

## Mail back a response to the given address.
## Data is a reference to an array or a scalar.
sub mailback {
   my($data, $subject, @rcpt) = @_;
   do_log('debug2', 'mail::mailback(%s, %s)', $subject, join(',', @rcpt));

   my $fh = smtp::smtpto("$Conf{'sympa'}", \@rcpt);
   printf $fh "To:  %s\n", join(",\n   ", @rcpt);
   print $fh "Subject: $subject\n";
   printf $fh "MIME-Version: %s\n", Msg(12, 1, '1.0');
   printf $fh "Content-Type: text/plain; charset=%s\n", Msg(12, 2, 'us-ascii');
   printf $fh "Content-Transfer-Encoding: %s\n", Msg(12, 3, '7bit');
   print $fh "\n";

   if (ref($data) eq 'SCALAR') {
      print $fh $$data;
   } elsif (ref($data) eq 'ARRAY') {
      print $fh @$data;
   }
   close($fh);
}

## send an archive file
sub mailarc {
   my($filename, $subject, @rcpt) = @_;
   do_log('debug2', 'mail::mailarc(%s, %s)', $subject, join(',', @rcpt));

   my($i);

   if (!open(IN, $filename)) {
      fatal_err("Can't send %s to %s: %m", $filename, join(',', @rcpt));
   }
   my($fh) = &smtp::smtpto($Conf{'request'}, \@rcpt);
   printf $fh "To: %s\n", join(",\n   ", @rcpt);
   print $fh "Subject: $subject\n";
   printf $fh "MIME-Version: %s\n", Msg(12, 1, '1.0');
   printf $fh "Content-Type: text/plain; charset=%s\n", Msg(12, 2, 'us-ascii');
   printf $fh "Content-Transfer-Encoding: %s\n", Msg(12, 3, '7bit');
   print $fh "\n";
   print $fh $i while ($i = <IN>);
   close($fh);
}


## send welcome,  bye, expire removed or reminder message 
sub mailfile {
   my($action, $listname, $default_subject, $default_body, $rcpt, $gecos) = @_;
   do_log('debug2', 'mail::mailfile(%s, %s, %s)', $listname, $rcpt, $gecos);

   my ($is_mime,$filename,$return_path);

   ## Return-path may be owner or unique
   if ($Conf{'return_path'} eq 'unique') {
       $return_path = 'something user $rcpt';
   }else {
       $return_path = "$listname-owner\@$Conf{'host'}"; 
   }
   
   my($fh) = &smtp::smtpto($return_path, \$rcpt);

   if (open(IN, "$listname/$action.mime")) {
       $filename="$listname/$action.mime";
       $is_mime=1;
   }elsif (open(IN, "$listname/$action")) {
       $filename="$listname/$action";
   }elsif (open(IN, "$action.mime")) {
       $filename="$action.mime";
       $is_mime=1;
   }elsif (open(IN, "$action")) {
       $filename="$action";
   }

   printf $fh "To: %s\n", join(",\n   ", $rcpt);

   unless ($is_mime){
       print $fh "From: $listname-request\@$Conf{'host'}\n"; 
       print $fh "Subject: $default_subject\n";
       printf $fh "MIME-Version: %s\n", Msg(12, 1, '1.0');
       printf $fh "Content-Type: text/plain; charset=%s\n", Msg(12, 2, 'us-ascii');
       printf $fh "Content-Transfer-Encoding: %s\n", Msg(12, 3, '7bit');
       print $fh "\n";
   }

   if ($filename) {
       while (<IN>){
	   s/\[listname\]/$listname/g;
	   s/\[subscriber_email\]/$rcpt/g;
	   s/\[email_subscriber\]/$rcpt/g;
	   s/\[subscriber_gecos\]/$gecos/g;
	   s/\[sympa_email\]/$Conf{'sympa'}/g;
	   s/\[sympa_host\]/$Conf{'host'}/g;
	   print $fh $_ ;
       }
       close IN;
   }else{
       print $fh $default_body;
   }
       
   close ($fh);
   return 1
}
1;








